' TITLE:
'  Mix 'n' Match
'
' AUTHOR:
'  Prospero / Firestorm Productions
'  http://www.firestormproductions.co.uk/
'
' WORKTIME:
'  10:50 to 12:15 on 24/11/07
'  15:30 to 16:02 on 24/11/07
'
' OBJECTIVE:
'  You are presented with a grid of hidden symbols. Click one and it
'  will flip over, allowing you to see it. Click a second and try to
'  match the first. Matches stay visible, but if they are different,
'  they become hidden again. Match all the pairs to win!
'
'===============================================================================

SuperStrict

Framework BRL.GLMax2D
Import BRL.FreeTypeFont
Import BRL.PNGLoader
Import BRL.Retro

Const DEBUG:Byte = True

Const SCALE:Byte = 2
Global g_height:Int = 320 * SCALE
Global g_width:Int = 240 * SCALE

Graphics g_height, g_width, 0

SeedRnd MilliSecs()
SetBlend ALPHABLEND
HideMouse





'Fonts
Global fntMain:TImageFont = LoadImageFont("fonts/supertext.ttf", 10, False)
Global fntSymbol:TImageFont  = LoadImageFont("fonts/iconbittwo.ttf", 20, False)

'Graphics
Global gfxPointer:TImage = LoadImage("graphics/pointer.png")
Global gfxHiddenTile:TImage = LoadImage("graphics/tile-hidden.png")

'Variables
Global i:Int, j:Int			'Loop controllers
Global tile:Byte[6, 6]			'The playfield
Global known:Byte[6, 6]		'Whether tiles are known or not
Global tilecol:String[18]		'Tile colours
Global mx:Int, my:Int			'Mouse position
Global sx1:Int, sy1:Int		'First selected tile
Global sx2:Int, sy2:Int		'Second selected tile
Global sec_x:Byte, sec_y:Byte	'Mouse hover tile
Global timer:Int				'Timer after second selection
Global won:Byte				'Did we win this board yet?




SetColor 255, 255, 255
SetScale SCALE, SCALE

NewColours()
NewTiles()

sx1 = -1 ; sy1 = -1
sx2 = -1 ; sy2 = -1

Repeat
	'** MAIN LOOP **
	Cls
	
	mx = MouseX()
	my = MouseY()
	
	HandleMouse()
	HandleTimer()
	
	SetBlend ALPHABLEND
	
	SetImageFont fntMain
	SetColor 255, 255, 255
	DrawCentredText "Mix 'n' Match!", 160, 3
	DrawCentredText "Hit F1 to reshuffle the board, or Q to quit.", 160, 225
	
	If won Then DrawCentredText "You won! Congratulations!", 160, 200
	
	If KeyHit(KEY_F1) Then
		NewColours()
		NewTiles()
	EndIf
	
	SetBlend MASKBLEND
	SetImageFont fntSymbol
	DrawTiles(97, 50)

	DrawImage gfxPointer, mx, my
		
	Flip
Until KeyDown(KEY_Q) Or AppTerminate()



Function HandleMouse()
'PURPOSE - Handles mouse position on the grid, mouse clicks, etc.
	If mx >= 166 And mx <= 466 And my >= 76 And my <= 376Then
		sec_x = (mx - 166) / 50
		sec_y = (my - 76) / 50

		If MouseHit(1) And timer = 0 Then
			If sx1 = -1 Then
				sx1 = sec_x
				sy1 = sec_y
			Else
				If Not (sec_x = sx1 And sec_y = sy1) Then
					sx2 = sec_x
					sy2 = sec_y
					
					timer = 60
					CheckMatch()
					won = CheckWin()
				EndIf
			EndIf
		EndIf
	EndIf
End Function




Function HandleTimer()
'PURPOSE - Updates the pick timer
	If timer > 0 Then
		timer :- 1
		If timer = 0 Then
			sx1 = -1 ; sy1 = -1
			sx2 = -1 ; sy2 = -1
		EndIf
	EndIf
End Function




Function CheckMatch()
'PURPOSE - Checks if both revealed tiles are a match. If they are, permanently show them
	If tile[sx1, sy1] = tile[sx2, sy2] Then
		known[sx1, sy1] = True
		known[sx2, sy2] = True
	EndIf
End Function




Function CheckWin:Byte()
'PURPOSE - Checks if all tiles are known. If they are, we won!
	Local is_won:Byte = True
	
	For i = 0 To 5
		For j = 0 To 5
			If known[i, j] = False Then is_won = False
		Next
	Next
	
	Return is_won
End Function




Function DrawScaledImage(img:TImage, _x:Int, _y:Int, _centre:Byte = False)
'PURPOSE - Draws an image to a scaled location on the screen
	If _centre Then
		DrawImage img, _x*SCALE - ImageWidth(img)/2, _y*SCALE - ImageHeight(img)/2
	Else
		DrawImage img, _x*SCALE, _y*SCALE
	EndIf
End Function




Function DrawScaledText(_str:String, _x:Int, _y:Int)
'PURPOSE - Draws text to a scaled location on the screen
	DrawText _str, _x*SCALE, _y*SCALE
End Function

Function DrawCentredText(_str$, _x:Int, _y:Int)
'PURPOSE - Draws a string of text horizontally and vertically centred at the given X location
	DrawText _str, Abs(_x-(TextWidth(_str)/2)) * SCALE, Abs(_y-(TextHeight(_str)/2)) * SCALE
End Function



Function NewColours()
'PURPOSE - Creates a colour for each of the tiles
	Local s$
	
	For j = 0 To 17
		s = RandHexColor(200)
		tilecol[j] = s
	Next
End Function



Function NewTiles()
'PURPOSE - Generates a new field of shuffled tiles
	Local idx:Byte = 0
	Local swapx1:Byte, swapx2:Byte
	Local swapy1:Byte, swapy2:Byte
	Local temp:Byte
	
	won = False
	
	'Create unshuffled playfield with 18 pairs
	For i = 0 To 5
		For j = 0 To 5
			'Reset known tiles while we're here
			known[i, j] = False
			
			tile[i, j] = idx
			idx :+ 1 ; If idx >= 18 Then idx = 0
		Next
	Next
	
	'Shuffle the playfield a bit
	For i = 1 To 500
		Repeat
			swapx1 = Rand(0, 5) ; swapy1 = Rand(0, 5)
			swapx2 = Rand(0, 5) ; swapy2 = Rand(0, 5)
		Until swapx1 <> swapx2 And swapy1 <> swapy2
		
		temp = tile[swapx1, swapy1]
		tile[swapx1, swapy1] = tile[swapx2, swapy2]
		tile[swapx2, swapy2] = temp
	Next
End Function



Function DrawTiles(_x:Int, _y:Int)
'PURPOSE - Draws the tiles starting from the given co-ordinates
	For i = 0 To 5
		For j = 0 To 5
			If (i = sx1 And j = sy1) Or (i = sx2 And j = sy2) Or known[i, j] Then
				SetHexColor(tilecol[tile[i, j]])
				DrawCentredText Chr(97 + tile[i, j]), _x + i*25, _y + j*25
			Else
				SetColor 191, 191 ,191
				DrawScaledImage gfxHiddenTile, _x-6 + i*25, _y-4 + j*25, True
			EndIf
		Next
	Next
End Function



Function SetHexColor(_hex:String)
'PURPOSE - Sets the drawing colour to the given hexadecimal value
	Local r:Int, g:Int, b:Int
	
	r = Int("$" + Left(_hex, 2))
	g = Int("$" + Mid(_hex, 3, 2))
	b = Int("$" + Right(_hex, 2))
	
	SetColor r, g, b
End Function



Function RandHexColor:String(_bright:Int)
'PURPOSE - Creates a random hexadecimal colour code
	Local color:String
	Local bright:Short
	Local digit:Byte
	Local r:Int, g:Int, b:Int

	Repeat
		'Create color code
		color = ""
		For i = 1 To 6
			digit = Rand(0, 15)
			If digit >= 0 And digit <= 9 Then
				color :+ digit
			Else
				Select digit
					Case 10 color :+ "A"
					Case 11 color :+ "B"
					Case 12 color :+ "C"
					Case 13 color :+ "D"
					Case 14 color :+ "E"
					Case 15 color :+ "F"
				End Select
			EndIf
		Next
		
		'Ensure the colour is not too dark - add r, g and b values
		r = Int("$" + Left(color, 2))
		g = Int("$" + Mid(color, 3, 2))
		b = Int("$" + Right(color, 2))
		bright = r + g + b
		
	Until bright > _bright

	Return color
End Function